--- chan_sip.c  2005-12-05 01:01:17.000000000 +0100
+++ chan_sip.c  2006-01-17 15:37:00.000000000 +0100
@@ -11045,6 +11045,7 @@
 #endif
                if (errno != ECONNREFUSED)
                        ast_log(LOG_WARNING, "Recv error: %s\n", strerror(errno));
+               siplockcounter = 0;
                return 1;
        }
        if (res == sizeof(req.data)) {
@@ -11070,6 +11071,7 @@

        if (req.headers < 2) {
                /* Must have at least two headers */
+               siplockcounter = 0;
                return 1;
        }

@@ -11079,6 +11081,16 @@
        ast_mutex_lock(&netlock);
        p = find_call(&req, &sin, req.method);
        if (p) {
+               if ( siplockcounter >= MAX_LOCK_RETRIES ) {
+                   ast_log(LOG_DEBUG, "Failed to grab lock. Ending loop, preventing deadlock...\n");
+                   siplockcounter = 0;
+                   sip_destroy(p);
+                   ast_mutex_unlock(&netlock);
+                   if (recount)
+                       ast_update_use_count();
+
+                   return 1;
+               }
                /* Go ahead and lock the owner if it has one -- we may need it */
                if (p->owner && ast_mutex_trylock(&p->owner->lock)) {
                        ast_log(LOG_DEBUG, "Failed to grab lock, trying again...\n");
@@ -11086,6 +11098,7 @@
                        ast_mutex_unlock(&netlock);
                        /* Sleep infintismly short amount of time */
                        usleep(1);
+                       (siplockcounter)++;
                        goto retrylock;
                }
                memcpy(&p->recv, &sin, sizeof(p->recv));
@@ -11108,6 +11121,7 @@
        ast_mutex_unlock(&netlock);
        if (recount)
                ast_update_use_count();
+       siplockcounter = 0;

        return 1;
 }
